#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#define CMDLINE
#include "cmdline.h"

#define       STR_SIZE        256

static const char sz_usage[] = "usage: sogen -i <inc_file> -n <struct_name> [-k <key_val>] [-K <key_string>] [-s <body_suffix>]\n";
static const char sz_help[] = "usage: sogen -i <inc_file> -n <struct_name> [-k <key_val>] [-K <key_string>] [-s <body_suffix>]\n\
Structure opetare code generator\n\
    -i <inc_file>    input include file. (required)\n\
    -n <struct_name> structre name(case sensitive). (required)\n\
    -k <key_val>     primary key type `value'\n\
    -K <key_string>  primary key type `string'\n\
    -s <body_suffix> body suffix of the filename(default:_mgr)\n\
    -h               produce this help message\n\
    -V               report version\n\
";
static const char sz_version[] = "version 1.0\n";

static char string_inc_file[STR_SIZE] = "";
static char string_struct_name[STR_SIZE] = "";
static char string_key_val[STR_SIZE] = "";
static char string_key_string[STR_SIZE] = "";
static char string_body_suffix[STR_SIZE] = "_mgr";

/* prototype decl. */
static void print_usage( void );
static void print_help( void );
static void print_version( void );

static void
print_usage( void )
{
	printf( sz_usage );
	exit( OPT_USAGE );
}

static void
print_help( void )
{
	printf( sz_help );
	exit( OPT_HELP );
}

static void
print_version( void )
{
	printf( sz_version );
	exit( OPT_VERSION );
}

static int
inc_fence_gurad( int i, int argc )
{
	i++;
	if ( i >= argc ) {
		fprintf( stderr, "too few argments\nTry sogen -h\n" );
		exit( OPT_ERROR_INVALID_OPT );
	}
	return i;
}

static int
is_string_int( const char *s )
{
	size_t i;
	size_t len = strlen( s );
	for ( i=0; i < len ; i++ ) {
		if ( !isdigit( s[i] ) ) {
			if ( i == 0 && s[i] == '-' ) continue;
			return 0;
		}
	}
	return 1;
}

void
cmdline_parse( int argc, char *argv[] )
{
	int i;
	/* flags for requied option(s) */
	int string_inc_file_given = 0;
	int string_struct_name_given = 0;

	/* usage */
	if ( argc == 1 ) {
		print_usage();
 	}

	if ( argc == 2 ) {
		if ( strcmp( argv[1], "-h" ) == 0 ) {
			print_help();
		} else if ( strcmp( argv[1], "-V" ) == 0 ) {
			print_version();
		}
	}

	/* all options scan */
	for( i=1; i < argc ; i++ ) {
		if ( !strcmp( argv[i], "-i" ) ) {
			i = inc_fence_gurad( i, argc );
			strcpy( string_inc_file, argv[i] );
			string_inc_file_given = 1;
		} else if ( !strcmp( argv[i], "-n" ) ) {
			i = inc_fence_gurad( i, argc );
			strcpy( string_struct_name, argv[i] );
			string_struct_name_given = 1;
		} else if ( !strcmp( argv[i], "-k" ) ) {
			i = inc_fence_gurad( i, argc );
			strcpy( string_key_val, argv[i] );
		} else if ( !strcmp( argv[i], "-K" ) ) {
			i = inc_fence_gurad( i, argc );
			strcpy( string_key_string, argv[i] );
		} else if ( !strcmp( argv[i], "-s" ) ) {
			i = inc_fence_gurad( i, argc );
			strcpy( string_body_suffix, argv[i] );
		} else {
			/* error */
			fprintf( stderr, "option `%s' is invalid.\n", argv[i] );
			exit( OPT_ERROR_INVALID_OPT );
		}
	}

	/* check requied option(s) */
	if ( string_inc_file_given == 0 ) {
		fprintf( stderr, "option `-i' is required.\n");
		exit( OPT_ERROR_INVALID_OPT );
	}
	if ( string_struct_name_given == 0 ) {
		fprintf( stderr, "option `-n' is required.\n");
		exit( OPT_ERROR_INVALID_OPT );
	}

}

const char *
cmdline_get_string_inc_file( void )
{
	return string_inc_file;
}

const char *
cmdline_get_string_struct_name( void )
{
	return string_struct_name;
}

const char *
cmdline_get_string_key_val( void )
{
	return string_key_val;
}

const char *
cmdline_get_string_key_string( void )
{
	return string_key_string;
}

const char *
cmdline_get_string_body_suffix( void )
{
	return string_body_suffix;
}


